exports.SYNC_state = SYNC_state;
exports.SYNC_update = SYNC_update;
exports.SYNC_create = SYNC_create;

var jQuery = require("../util").jQuery();
var get_utc = require("../util").get_utc;
var API = require("../const").API();
var CommInfo = require("../common").CommInfo();
var localStorage = require("sdk/simple-storage").storage;
var urlencode = require("../util").urlencode;
var get_clientid = require("../util").get_clientid;
var pre_process_local = require("./merge").pre_process_local;
var bookmark_merge_ctrl = require("./merge").bookmark_merge_ctrl;
var BookmarkSyncState = require("./struct").BookmarkSyncState();
var sync_node_create = require("./merge").sync_node_create;
var sync_node_update = require("./merge").sync_node_update;
var BookmarkInfo = require("./control").BookmarkInfo();

var track_event = require("../track").track_event;
var print_msg = require("../util").print_msg;
BOOKMARK_SID = {
	current:0,
	ongoing:0
};

/*
	Get sync state
*/
function SYNC_state()
{
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
	track_event({
		category:'general',
		action:'BookmarkSync',
		label:'trigger',
		value:1
	});	
	
    //console.log("get state");
	var data={'token':CommInfo.token};
	var data_encode = urlencode(data);

	jQuery.ajax({
		url:API.sync_state(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		headers:{"ClientVersion":"firefoxExtv1.0"},
		success:SYNC_state_callback,
		error:function(){
			//console.log("get state error.");
			BookmarkInfo.ongoing = 0;
			track_event({
				category:'general',
				action:'BookmarkSync',
				label:'fail',
				value:1
			});				
		}
	});
}

/*
	Callback of API_sync_state.
	Params:
		xhr: XMLHttpRequest
*/
function SYNC_state_callback(resp)
{
    //console.log("get state callback");
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
    //console.log(JSON.stringify(resp));
	if(resp.status == 0) 	//connect success
	{	
		//bookmark
		var latest_sid = resp.data.latest_firefox_sid;
		var	after_sid = CommInfo.after_sid();	
		BOOKMARK_SID.current = after_sid['bookmark'];
		BOOKMARK_SID.ongoing = after_sid['bookmark'];
				
		//when there is new data updated in server, get chunk
		if(after_sid['bookmark'] < latest_sid)
		{
			//clear buffer  *doubtful*
			print_msg('[getchunk] clear buffer!!!')
			BookmarkInfo.deletes.splice(0, BookmarkInfo.deletes.length);
			BookmarkInfo.updates.splice(0, BookmarkInfo.updates.length);
			
			SYNC_getchunk({type:'bookmark',latest_sid:latest_sid});
		}
		else
		{
			pre_process_local();
			//bookmark_sync_ctrl();
		}
	}
	else
	{
		//console.log('state error.');
		BookmarkInfo.ongoing = 0;
		track_event({
			category:'general',
			action:'BookmarkSync',
			label:'fail',
			value:1
		});			
	}
}

/*
	Send get chunk request.
	Params:
		context:sync item information, json dict
*/
function SYNC_getchunk(context)
{	
    //console.log("get chunk");
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
	var data = {};
	var token = CommInfo.token;
	var after_sid = BOOKMARK_SID.ongoing;//CommInfo.after_sid()[context.type];
	var latest_sid = context.latest_sid;
	var limit = latest_sid - after_sid;
	var type = get_type(context);
	
	data.token=token;
	data.after_sid = after_sid;
	data.limit = limit>20?20:limit;
	data.type = type;
	
	if(BOOKMARK_SID.current == 0) {
		data.no_deleted = 1;
	}
	
	data_encode = urlencode(data);
	
	jQuery.ajax({
		url:API.sync_getchunk(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		success:SYNC_getchunk_callback,
		error:function(){
			//console.log("get chunk error.");
			BookmarkInfo.ongoing = 0;
			track_event({
				category:'general',
				action:'BookmarkSync',
				label:'fail',
				value:1
			});				
		}
	});
}

/*
	Callback of getchunk.
	This function save the obj get from server to local storage,
	if get chunk is not finish, getchunk will be called again.
	
	Params:
		xhr: XMLHttpRequest.
*/
function SYNC_getchunk_callback(resp)
{
        //console.log("get chunk callback");
		var BookmarkInfo = require("./control").BookmarkInfo();	
		
		if (CommInfo.is_login == false) {
			BookmarkInfo.ongoing = 0;
			BookmarkInfo.local = null;		
			return;
		}
		
		var time_current = get_utc()/1000;
		BookmarkInfo.lastSyncTime = time_current;
	
		if(resp.status == 0) 	//connect success
		{	
			var latest_sid = resp.data.latest_sid;
			var chunk_latest_sid = resp.data.chunk_latest_sid;
			var updated_objs = resp.data.updated_objs;
			
			BookmarkInfo.deletes = BookmarkInfo.deletes.concat(resp.data.deleted_ids);
			BookmarkInfo.updates = BookmarkInfo.updates.concat(updated_objs);

			//more data need receive.
			if(chunk_latest_sid < latest_sid)
			{
				BOOKMARK_SID.ongoing = chunk_latest_sid;
				SYNC_getchunk({type:'bookmark',latest_sid:latest_sid});
			}
			else
			{
				BOOKMARK_SID.ongoing = chunk_latest_sid;
				BOOKMARK_SID.current = chunk_latest_sid;
				
				var after_sid = CommInfo.after_sid();
				
				//get bookmark after_sid
				after_sid['bookmark'] = chunk_latest_sid;
				
				//save after_sid
				var after_sid_str = localStorage['DolphinBrowserAfterSid'];
				var after_sid_json;
				if(after_sid_str != null && after_sid_str != "")
				{
					after_sid_json = JSON.parse(after_sid_str);
				}
				else
				{
					after_sid_json={};
				}
				
				after_sid_json[CommInfo.user_name] = after_sid;
				localStorage['DolphinBrowserAfterSid'] = JSON.stringify(after_sid_json);
				
				//bookmark data receive finish, start merge process.
				bookmark_merge_ctrl();
			}
		}
		else
		{
			//console.log("SYNC_getchunk_callback error.");
			BookmarkInfo.ongoing = 0;
			track_event({
				category:'general',
				action:'BookmarkSync',
				label:'fail',
				value:1
			});				
		}
}

/*
	Update the device current open tabs 
	Params:
		sync_id:user sync_id
*/
function SYNC_update(data)
{
    print_msg('update');
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
    //console.log(JSON.stringify(data));
	var data_encode = urlencode({'token':CommInfo.token,'data':data, "device_id":get_clientid(true)});
	jQuery.ajax({
		url:API.sync_update(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		success:SYNC_update_callback,
		error:function(){
			//console.log("update error.");
			//BookmarkInfo.ongoing = 0;
			track_event({
				category:'general',
				action:'BookmarkSync',
				label:'update fail',
				value:1
			});
			sync_node_update();
		}
	});
}	

function SYNC_update_callback(resp){
	//console.log("update callback");
	//console.log(JSON.stringify(resp));
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
	var local = BookmarkInfo.local;
    if(resp.status == 0) 	//connect success
	{
		if(resp.data && resp.data.length > 0 ) {
			for(var idx in resp.data) {
				//do something here.
				var data = resp.data[idx];
				var local_id = data.cid;
				if(!local[local_id]){
					continue;
				}
				if(local[local_id].state == BookmarkSyncState.REMOVE) {
					delete local[local_id];
				}else{
					local[local_id].state = BookmarkSyncState.SYNCED;				
				}
				setAfterSid(data.sid);
			}
		}
	}
	else
	{
		//console.log("update callback error.");
		BookmarkInfo.ongoing = 0;
		track_event({
			category:'general',
			action:'BookmarkSync',
			label:'fail',
			value:1
		});			
	}
	sync_node_update();
}

/*
	Create a new tab id, this means first sync for this device.
	Params:
		-data:  array list.
*/
function SYNC_create(data)
{
    //console.log("create");
    print_msg("create");
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
	var data_encode = urlencode({'token':CommInfo.token,'data':data, "device_id":get_clientid(true)});
	jQuery.ajax({
		url:API.sync_create(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		success:SYNC_create_callback,
		error:function(){
			//console.log("create error.");
			BookmarkInfo.ongoing = 0;
			track_event({
				category:'general',
				action:'BookmarkSync',
				label:'fail',
				value:1
			});				
		}
	});
}

/*
	Save sync id to local storage
	Params:
		xhr: XMLHttpRequest.
*/
function SYNC_create_callback(resp)
{	
    //console.log("create callback");
	var BookmarkInfo = require("./control").BookmarkInfo();	
	
	if (CommInfo.is_login == false) {
		BookmarkInfo.ongoing = 0;
		BookmarkInfo.local = null;		
		return;
	}
	
	var time_current = get_utc()/1000;
	BookmarkInfo.lastSyncTime = time_current;
	
	var local = BookmarkInfo.local;
    if(resp.status == 0) 	//connect success
	{	
		if(resp.data && resp.data.length > 0) {
			for(var id in resp.data) {
				var data = resp.data[id];
				var sync_id = data._id;
				var local_id = data.cid;
				if(local[local_id]){
					local[local_id].sid = sync_id;
					local[local_id].state = BookmarkSyncState.SYNCED;
					setAfterSid(data.sid);
				}			
			}			
		}
	}
	else
	{
		//console.log("create callback error.");
		BookmarkInfo.ongoing = 0;
		track_event({
			category:'general',
			action:'BookmarkSync',
			label:'fail',
			value:1
		});	
	}
	sync_node_create();
}


/*
	Get sync item type number
	Params:
		context: sync item information
*/
function get_type(context)
{
	if(context.type == 'bookmark')
		return 128;
	else if(context.type == 'tab')
		return 2;
	else if(context.type == 'history')
		return 4;
}


/* update sid
	@param {int}sid
	@return null
*/
function setAfterSid(sid){
	var BookmarkInfo = require("./control").BookmarkInfo();
	if(sid > BookmarkInfo.latest_sid)
		BookmarkInfo.latest_sid = sid;
	return ;
}