exports.AddBookmarkListener = AddBookmarkListener;

var BookmarkSyncStorage = require("./storage").BookmarkSyncStorage();
var chrome = {bookmarks:{}};
var bookmarkAdaptor = require("./bookmarkAdaptor");
var BookmarkInfo = require("./control").BookmarkInfo();
var BookmarkSyncCtrl = require("./control").BookmarkSyncCtrl();
var BookmarkSync = require("./control").BookmarkSync;
var CommInfo = require("../common").CommInfo();
var BookmarkSyncState = require("./struct").BookmarkSyncState();
var print_msg = require("../util").print_msg
chrome.bookmarks.onMoved = bookmarkAdaptor.onMoved();
chrome.bookmarks.onCreated = bookmarkAdaptor.onCreated();
chrome.bookmarks.onRemoved = bookmarkAdaptor.onRemoved();
chrome.bookmarks.onChanged = bookmarkAdaptor.onChanged();
chrome.bookmarks.onChildrenReordered = bookmarkAdaptor.onChildrenReordered();

var timers = require("sdk/timers");
var setTimeout = timers.setTimeout;
var default_timeout = 500;
var autoTimer = null;
function autoSync(){
	print_msg('set autoSync timer!');
	if(autoTimer){
		timers.clearTimeout(autoTimer);
		autoTimer = setTimeout(BookmarkSync, 5000);
	}else{
		autoTimer = setTimeout(BookmarkSync, 5000);
	}
}
function AddBookmarkListener(){
	//console.log("bookmark listener adding...");

	chrome.bookmarks.onCreated.addListener(
			function(id, createInfo) {
				//console.log("detect bookmark create.");
				var BookmarkInfo = require("./control").BookmarkInfo();
				////console.log(BookmarkInfo.ongoing);
				//console.log(BookmarkInfo.ongoing, CommInfo.is_login);
				if(BookmarkInfo.ongoing == 0 && CommInfo.is_login)  {
					//sync start
					//console.log("start sync.");
					setTimeout(autoSync,default_timeout);
				}		
			}	
	);
	chrome.bookmarks.onChanged.addListener(
		function(id,changeInfo){
			//console.log("detect bookmark changed.");
			var BookmarkInfo = require("./control").BookmarkInfo();
			//console.log(BookmarkInfo.ongoing, CommInfo.is_login);
			if(BookmarkInfo.ongoing == 0 && CommInfo.is_login) {
				//sync start
				//console.log("detect bookmark change.");
				//console.log(JSON.stringify(changeInfo));
				setTimeout(autoSync,default_timeout);
			}	
		}
	);

	/*
		{
			'index':0,
			'oldindex':1,
			'oldParentId':"20",
			'parentId':"20"
		}
	*/
	chrome.bookmarks.onMoved.addListener(
		function(id, moveInfo) {
			//console.log("detect bookmark moved");
			var BookmarkInfo = require("./control").BookmarkInfo();
			//console.log(BookmarkInfo.ongoing, CommInfo.is_login);
			if(BookmarkInfo.ongoing == 0 && CommInfo.is_login) {
				var bk_data = BookmarkSyncStorage.load();
				if(bk_data.hasOwnProperty(id))
				{
					if(moveInfo.oldParentId != moveInfo.parentId){
						bk_data[id].pid = moveInfo.parentId;
					}
					
					//mark it's parent to dirty and state to MOVE
					bk_data[moveInfo.parentId].dirty=1;
					bk_data[id].state = BookmarkSyncState.MOVE;
					
					BookmarkSyncStorage.save(bk_data);
					//console.log('move bookmarks'+id);
				}
				//sync start
				//console.log("detect bookmark move.");
				setTimeout(autoSync,default_timeout);
			}	
		}
	);

	chrome.bookmarks.onRemoved.addListener(
			function(id, removeInfo) {
				//console.log("detect bookmark removed");
				var BookmarkInfo = require("./control").BookmarkInfo();
				//console.log(BookmarkInfo.ongoing == 0);
				if(BookmarkInfo.ongoing == 0 && CommInfo.is_login) {
					//sync start
					//console.log("detect bookmark remove.");
					setTimeout(autoSync,default_timeout);
				}				
			}
		);

	/*
	 	reorderInfo is a list of id of children.
	 */
	chrome.bookmarks.onChildrenReordered.addListener(
		function (id, reorderInfo) {
			var BookmarkInfo = require("./control").BookmarkInfo();
			if(BookmarkInfo.ongoing == 0 && CommInfo.is_login) {
				//sync start
				if(reorderInfo){
					var bk_data = BookmarkSyncStorage.load();
					bk_data[id].dirty = 1;
					BookmarkSyncStorage.save(bk_data);
					//console.log('children reorder bookmarks'+id);
					//console.log("detect bookmark children reorder.");
					setTimeout(autoSync,default_timeout);
				}
			}	
		}	
	);	
	//console.log("bookmark listener added...");
}
