exports.inject_init = inject_init;
exports.executeScript = executeScript;

var pageMod = require("sdk/page-mod");
var data = require("sdk/self").data;
var CommInfo = require("./common").CommInfo();
var pushMod = require("./push/push")
var track_event = require("./track").track_event;
var print_msg = require("./util").print_msg;
function inject_init(){
    // inject baidu map
    pageMod.PageMod({
        include: [
        	"http://map.baidu.com.cn/*",
        	"http://maps.baidu.com.cn/*",
        	"http://ditu.baidu.com.cn/*",
        	"http://map.baidu.com/*",
        	"http://maps.baidu.com/*",
        	"http://ditu.baidu.com/*",
        	"http://ditu.baidu.cn/*", 
        	"http://maps.baidu.cn/*",
        	"http://map.baidu.cn/"
        ],
        contentScriptFile: data.url("src/cs_baidu_map.js"),
        contentScriptOptions:{
            srcRoot:data.url("src")
        },
        onAttach: function(worker) {
            eventCtrl(worker);
        } 
    });

    //inject google map
    pageMod.PageMod({
        include: [
        	"http://ditu.google.cn/*",				
        	"http://ditu.google.com/*",
			"http://maps.google.com/*",
			"http://maps.google.com.ph/*",
			"http://maps.google.no/*",
			"http://maps.google.it/*",
			"http://maps.google.pl/*",
			"http://maps.google.ru/*",
			"http://maps.google.com.sg/*",
			"http://maps.google.ch/*",
			"http://maps.google.com.tw/*",
			"http://maps.google.co.uk/*",
			"http://maps.google.com.hk/*",	
			"http://maps.google.co.jp/*",
			"http://maps.google.co.kr/*",
			"https://ditu.google.com/*",
			"https://maps.google.com/*",
			"https://maps.google.com.ph/*",
			"https://maps.google.no/*",
			"https://maps.google.it/*",
			"https://maps.google.pl/*",
			"https://maps.google.ru/*",
			"https://maps.google.com.sg/*",
			"https://maps.google.ch/*",
			"https://maps.google.com.tw/*",
			"https://maps.google.co.uk/*",
			"https://maps.google.com.hk/*",
			"https://maps.google.co.jp/*",
			"https://maps.google.co.kr/*"
        ],
        contentScriptFile: data.url("src/cs_google_map.js"),
        contentScriptOptions:{
            srcRoot:data.url("src")
        },
        onAttach: function(worker) {
            eventCtrl(worker);
        }
    });
    
    //inject appstore
    pageMod.PageMod({
        include: ["http://itunes.apple.com/*", "https://itunes.apple.com/*"],
        contentScriptFile: data.url("src/cs_appstore2.js"),
        contentScriptOptions:{
            srcRoot:data.url("src")
        },
        onAttach: function(worker) {
            eventCtrl(worker);
        }
    });

    //inject shortcuts
    pageMod.PageMod({
        include: ["http://*", "https://*"],
        contentScriptFile: data.url("src/cs_shortcut.js"),
        contentScriptOptions:{
            srcRoot:data.url("src")
        },        
        onAttach: function(worker) {
            eventCtrl(worker);
        }
    });
}

function executeScript(cond, contentScript) {
	var curTab = require("sdk/tabs").activeTab;
	curTab.attach({
		contentScriptFile: require("sdk/self").data.url("src/"+contentScript.file),
		contentScriptOptions:{
			srcRoot:require("sdk/self").data.url("src"),
			code:contentScript.code
		}
	});
}

function eventCtrl(worker){
    worker.port.on("cs_msg", function(message){
        var type = message.type;
        var data = message.data;
        
        if(type == 'dev_data') {
            print_msg("dev_data");
            var devs = CommInfo.get_mobile_devs();
            //print_msg(JSON.stringify(devs));
            worker.port.emit("cs_msg_page", devs);
            
        }
        else if(type == 'buttonsetting') {
            print_msg("recv:buttonsetting");
            worker.port.emit("cs_msg_page", CommInfo.get_setting()['button']);
        }
        else if(type == 'map') {
        	if(!CommInfo.is_login) {
        		worker.port.emit("cs_msg_page", {status:1});
        		return;
        	}
        	var setting = CommInfo.get_setting();
        	if(!setting['button']) {
        		worker.port.emit("cs_msg_page", {status:2});
        		return;
        	}
        	
        	pushMod.push(data.dev_id, {title:data.title, url:data.url}, false, 1, 0);
        }
        else if (type == 'appstore') {
        	if(!CommInfo.is_login) {
        		worker.port.emit("cs_msg_page", {status:1});
        		return;
        	}
        	var setting = CommInfo.get_setting();
        	if(!setting['button']) {
        		worker.port.emit("cs_msg_page", {status:2});
        		return;
        	}
        	pushMod.push(data.dev_id, {title:data.title, url:data.url}, false, 1, 1);
        }
        else if(type == 'shortcuts') {
        	var keys = data;
        	print_msg('shortcuts');
        	if(CommInfo.is_login == true){
        		var dev_keys = CommInfo.get_device_shortcut();
        		for(var idx in dev_keys) {
        			//if(keys.length == dev_keys[idx].length) {
        				var match = true;
        				for(var id in keys) {
        					if(keys[id] != dev_keys[idx][id]) {
        						match = false;
        						break;
        					}
        				}
        				if(match) {
        					//print_msg("match"+idx);
        					pushMod.pushSend(idx);
							track_event({
								category:'other',
								action:'shortcutspush',
								label:'shortcutspush',
								value:1
							});	
        					break;
        				}
        			//}
        		}	
        	}
        }        
        else if(type == 'dolphin_clickTarget') {
        	var target = data;
			CommInfo.clickTarget = target;
        }
    });
}
