exports.get_sonar_cmd = get_sonar_cmd;
exports.checkGeolocationAPI = checkGeolocationAPI;

var API = require("../const").API();
var util = require("../util");
var jQuery = util.jQuery();
var CommInfo = require("../common").CommInfo();
var get_clientid = util.get_clientid;
var SonarPusher = require("../push/push_proxy").SonarPusher;
var timers = require("sdk/timers");
var setTimeout = timers.setTimeout;
var clearTimeout = timers.clearTimeout;
//var destory_loginWin = require("../main").destory_loginWin;
var login_init = require("./account").login_init
//var INJECT_CODE = require("../common").INJECT_CODE()
var extLocale  = require("../string").extLocale();
var get_user_info = require("./account").get_user_info
var print_msg = require("../util").print_msg;
const {Cc,Ci} = require("chrome");
/*
	Sonarlogin module.This module has some functions as follows:
	1. Get a phrase from voice server;
	2. Wait for a connect from phone.
	
	After connect from phone, sonar login panel will be closed, and coming into main panel,
	then tab sync will be started and do some initialization of login.
*/

var SonarConnectInfo = {
	receiver:null,
	timeout_id:0,
	timeout:2*60*1000, //2min 
	display_word:function(){},
	display_error:function(){},
	win_close:function(){},
	request:null,
	type:0
};

var TIMEOUT_MSG = extLocale["Phrase expired"];
var LOGIN_SUCCESS_MSG = "Login successful"; 

/*
	Create a instance of sonar push module.
*/
function sonar_init()
{
	/*
		if no sonar connect push receiver exists, create one.
	*/
	if(SonarConnectInfo.receiver == null)
	{
		var client_id = get_clientid(false);
		SonarConnectInfo.receiver = new SonarPusher({dev_id:client_id, handler:login_connect});
		SonarConnectInfo.receiver.login();
	}
	
	/*
		clear timeout for connect time expire.
	*/	
	clearTimeout(SonarConnectInfo.timeout_id);
	
	/*
		when a get phrase request is ongoing, stop it.
	*/
	if(SonarConnectInfo.request != null)
	{
		SonarConnectInfo.request.abort();
	}
}

/*
	Disconnect push module from server and free sonar pusher instance.
*/
function sonar_close()
{
	if(SonarConnectInfo.receiver != null)
	{
		SonarConnectInfo.receiver.disconnect();
		SonarConnectInfo.receiver = null;
	}
	clearTimeout(SonarConnectInfo.timeout_id);
	SonarConnectInfo.request = null;
}

/*
	Get a sonar phrase from server.
	Params:
		-show_word: handler for display word in popup panel.
		-show_error:handler for display error info.
		-win_close: window close function.
		-type:0 or 1, 0:first get word, 1: get another word.
*/
function get_sonar_cmd(show_word,show_error,win_close,type)
{	
	sonar_init();
	checkGeolocationAPI();
	SonarConnectInfo.display_word = show_word;
	SonarConnectInfo.display_error = show_error;
	SonarConnectInfo.win_close = win_close;
	SonarConnectInfo.type = type;
}

/*
	Call Geolocation API get location information and call get phrase API. 
*/
function checkGeolocationAPI()
{
    print_msg("get command");
    ////getCurrentPosition(get_location, get_failed);
    get_failed();
    //if(navigator.geolocation)
	//{
	//	navigator.geolocation.getCurrentPosition(get_location,get_failed);
	//}
	//else
	//{
		//navigator is supported by chrome,so there is no problem here
		//print_msg('Geo location is not supported');
	//}
}

function getCurrentPosition(success,error){
    var xpcomGeolocation = Cc["@mozilla.org/geolocation;1"].getService(Ci.nsIDOMGeoGeolocation);
    xpcomGeolocation.getCurrentPosition(success, error);
}

/*
	Get location failed
*/
function get_failed(code)
{
	get_location({coords:{latitude:0,longitude:0}});
}

/*
	Get latitude and longitude and send data to server
	Params:
		position:
*/
function get_location(position)
{
    print_msg("get position");
	var lat = position.coords.latitude;
	var lon = position.coords.longitude;
	var client_id = get_clientid(false);	
	
	var body = "location=["+lon+","+lat+"]&client_id="+client_id+"&type="+SonarConnectInfo.type;
	print_msg(body);
	SonarConnectInfo.request =  jQuery.ajax({
            url:API.sonar_command,
            contentType:"application/json",
            type:"post",
            data:body,
            success:get_command_resp_proc,
            error:get_word_fail          
        });
}

function get_command_resp_proc(msg)
{
	if (msg.status == 0)
	{
		CommInfo.sonar_cmd = msg.data.text;
	    print_msg("get sonar word" + msg.data.text);	
                
        SonarConnectInfo.display_word(CommInfo.sonar_cmd);
	
                        
		SonarConnectInfo.timeout_id = setTimeout(function(){
			SonarConnectInfo.display_word(TIMEOUT_MSG, true);
			},
			SonarConnectInfo.timeout
		);
                      
	}
	else
	{
        print_msg("get sonar word fail.");
		//where should have a error display, but in this version dont consider this condition.
		SonarConnectInfo.display_error();
		clearTimeout(SonarConnectInfo.timeout_id);
		//document.getElementById('voice_text').value = "service not available";
	}
}

function get_word_fail()
{
        clearTimeout(SonarConnectInfo.timeout_id);
        SonarConnectInfo.display_error();
        SonarConnectInfo.request = null;
}


function login_connect(resp)
{
	print_msg("sonar login successful.");
	connect_success(resp);
	sonar_close();
	SonarConnectInfo.win_close();
	//login_init();
        get_user_info();
}

function connect_success(resp)
{
	
	var login_type = resp.type;
		
	//save connect user info to common data structure.
	CommInfo.token = resp.token;
	CommInfo.user_name = resp.email;
	CommInfo.email = resp.email;
	CommInfo.nick_name = resp.nick_name;
	CommInfo.login_type = login_type;
	CommInfo.is_login = true;
	
	if(resp.region_domain != null) {
		CommInfo.region_domain = resp.region_domain;
	}
	if(resp.push_domain != null) {
		CommInfo.push_domain = resp.push_domain;	
	}
	
	/*
		when username passed from sonarlogin phone, it's like ray@gmail.com@goole,
		so, filter '@google' out.
	*/
	var tmp = CommInfo.user_name;
	var first_place = tmp.indexOf('@');
	var secnd_place = tmp.indexOf('@',first_place+1);
	if(secnd_place != -1)
	{
		var username = tmp.substring(0, secnd_place);
		CommInfo.user_name = username;
	}	
	
	/*
		if user nick_name is null or has no body, use user_name instand.
	*/
	if(CommInfo.nick_name == null || CommInfo.nick_name.length == 0)
	{
		CommInfo.nick_name = CommInfo.user_name;
	}
	if((typeof login_type) == 'string') {
		login_type = parseInt(login_type);
	}	
	switch(login_type)
	{
		case 0: //dolphin
		{
			CommInfo.login_typeName = 'Dolphin';
		}
		break;
		case 1: //sinaweibo
		{
			CommInfo.login_typeName = extLocale['Sinaweibo'];
		}
		break;
		case 10://google
		{
			CommInfo.login_typeName = 'Google';
		}
		break;	
		case 11://facebook
		{
			CommInfo.login_typeName = 'Facebook';
		}
		break;
		case 4://tencentQQ
		{
			CommInfo.login_typeName = extLocale['tencentQQ'];
		}
		break;
		default:
			CommInfo.login_typeName = 'Dolphin';				
	}
	
	//chrome.tabs.executeScript(null, {code: INJECT_CODE.SONAR_LOGIN, file: "cs_overlayer.js"});
	
	print_msg("login success");
}

var chrome = {
	tabs:{
		executeScript: require("../inject_cs").executeScript
	}
};
