exports.create_menus_page = create_menus_page;
exports.create_menus_image = create_menus_image;
exports.create_menus_selection = create_menus_selection;

exports.removeall_menu = removeall_menu;

var CommInfo = require("./common").CommInfo();
var cm = require("sdk/context-menu");
var push = require("./push/push").push;
var data = require("sdk/self").data;
var extLocale = require("string").extLocale();
var getQueryStringRegExp = require("./util").getQueryStringRegExp;

var menus_list = {
    menu_page: null,
    menu_image: null,
    menu_selection: null
};

function removeall_menu(){
   if(menus_list.menu_page != null){
       menus_list.menu_page.destroy();
       menus_list.menu_page = null;
   }

   if(menus_list.menu_image != null){
       menus_list.menu_image.destroy();
       menus_list.menu_image = null;
   }
   if(menus_list.menu_selection != null){
       menus_list.menu_selection.destroy();
       menus_list.menu_selection = null;
   }
}

function create_menus_page(devs) {

    if(menus_list.menu_page != null){
         menus_list.menu_page.destroy();
    }
    
    var len = devs.length;
    if(len == 0) {
         menus_list.menu_page = cm.Item({
             label: extLocale["Send to Dolphin"],
             data:"",
             image:data.url("src/images/logo16.png"),
             context:[
                 cm.URLContext(["http://*","https://*"]),
                 cm.PageContext()
             ],
             contentScript: 'self.on("click", function(node,data) { window.alert("'+ extLocale["You haven't logged in with Dolphin Connect in your Dolphin Browser."]+ '");})'
         });    
    }     
    else if(len == 1) {
         menus_list.menu_page = cm.Item({
             label: extLocale["Send to Dolphin"],
             data:devs[0].did,
             image:data.url("src/images/logo16.png"),
             context: [
                 cm.URLContext(["http://*","https://*"]),
                 cm.PageContext()
             ],    
             contentScript: 'self.on("click", function(node,data) { self.postMessage({dev_id:data,url:window.location.href,title:window.document.getElementsByTagName("title")[0].innerText});})',
             onMessage:function(info){
                push(info.dev_id,{title:info.title,url:info.url}, false, 1); 
             }
         });     
    }
    else {
       var dev_items = [];
       for(var idx in devs) {
    	   var item_info = {label:devs[idx].deviceName, data:devs[idx].did};
           dev_items.push(cm.Item(item_info));
       }

       menus_list.menu_page = cm.Menu({
           label:extLocale["Send to Dolphin"],
           image:data.url("src/images/logo16.png"),
           context: [
               cm.URLContext(["http://*","https://*"]),
               cm.PageContext()
           ],    
           contentScript: 'self.on("click", function(node,data) { self.postMessage({dev_id:data,url:window.location.href,title:window.document.getElementsByTagName("title")[0].innerText});})',
           onMessage:function(info) {
                push(info.dev_id,{title:info.title,url:info.url}, false, 1); 
           },
           items: dev_items          
       }); 
    } 
}

function create_menus_image(devs) {
    if(menus_list.menu_image != null) {
        menus_list.menu_image.destroy();
    }
    var len = devs.length;
    if(len == 0) {
        menus_list.menu_image = cm.Item({
            label: extLocale["Send image to Dolphin"],
            data:"",
            image:data.url("src/images/logo16.png"),
            context:[
                cm.URLContext(["http://*","https://*"]),
                cm.SelectorContext("img")
            ],    
            contentScript: 'self.on("click", function(node,data) { window.alert("'+ extLocale["You haven't logged in with Dolphin Connect in your Dolphin Browser."]+ '");})'
        });     
    } 
    else if(len == 1) {
        menus_list.menu_image = cm.Item({
            label: extLocale["Send image to Dolphin"],
            data:devs[0].did,
            image:data.url("src/images/logo16.png"),
            context:[
                cm.URLContext(["http://*","https://*"]),
                cm.SelectorContext("img")
            ],    
            contentScript: 'self.on("click", function(node,data) { var link=null; var p= node.parentNode; if(p) { link=p.href; } self.postMessage({dev_id:data,srcUrl:node.src,title:"", linkUrl:link, pageUrl:window.location.href});})',
            onMessage:function(info){
				var src = image_src_parse(info);	
                push(info.dev_id,{title:info.title,url:src}, false, 2);
            }
        });     
    }
    else {
        var dev_items = [];
        for(var idx in devs) {
            dev_items.push(cm.Item({label:devs[idx].deviceName, data:devs[idx].did}));
        }

        menus_list.menu_image = cm.Menu({
            label:extLocale["Send image to Dolphin"],
            image:data.url("src/images/logo16.png"),
            context:[
                cm.URLContext(["http://*","https://*"]),
                cm.SelectorContext("img")
            ],    
            contentScript: 'self.on("click", function(node,data) { var link=null; var p= node.parentNode; if(p) { link=p.href; } self.postMessage({dev_id:data,srcUrl:node.src,title:"", linkUrl:link, pageUrl:window.location.href});})',
            onMessage:function(info) {
            	var src = image_src_parse(info);
                push(info.dev_id,{title:info.title,url:src}, false, 2); 
            },
            items: dev_items          
        }); 
    } 
}

function image_src_parse(info) {
	var src= info.srcUrl;
	
	if(src.indexOf("data:") == 0 && info.linkUrl) {
		var imgurl = getQueryStringRegExp(info.linkUrl,"imgurl");
		src = !imgurl?src:imgurl;
	}
	else if(info.pageUrl.indexOf("image.baidu.") != -1 || 
		info.pageUrl.indexOf("images.baidu.") != -1 || 
		info.pageUrl.indexOf("tupian.baidu.") != -1 || 
		info.pageUrl.indexOf("pic.baidu.") != -1 || 
		info.pageUrl.indexOf("picture.baidu.") != -1) 
	{
		//baidu image
		if(CommInfo.clickTarget != null){
			src = CommInfo.clickTarget;
		}
		else if(info.linkUrl != null) {
			src = info.linkUrl;
		}
	}
	
	return src;	
}

function create_menus_selection(devs) {
    if(menus_list.menu_selection != null) {
        menus_list.menu_selection.destroy();
    }
    var len = devs.length;
    if(len == 0) {
        menus_list.menu_selection = cm.Item({
            label: extLocale["Send to Dolphin"],
            data:"",
            image:data.url("src/images/logo16.png"),
            context: [
                cm.URLContext(["http://*","https://*"]),
                cm.SelectionContext()
            ],    
            contentScript: 'self.on("click", function(node,data) { window.alert("'+ extLocale["You haven't logged in with Dolphin Connect in your Dolphin Browser."]+ '");})'
        });     
    } 
    else if(len == 1) {
        menus_list.menu_selection = cm.Item({
            label: extLocale["Send to Dolphin"],
            data:devs[0].did,
            image:data.url("src/images/logo16.png"),
            context: [
                cm.URLContext(["http://*","https://*"]),
                cm.SelectionContext()
            ],    
            contentScript: 'self.on("click", function(node,data) { var text = ""; if(node.tagName.toLowerCase()=="input") { text = node.value.substring(node.selectionStart, node.selectionEnd);}else{ text=window.getSelection().toString(); } if(text.length > 1000) { alert(decodeURIComponent("%E4%B8%8D%E8%83%BD%E8%B6%85%E8%BF%871000%E5%AD%97%E7%AC%A6%E3%80%82")); return;}self.postMessage({dev_id:data,url:"",title:text});})',
            onMessage:function(info){
                push(info.dev_id,{title:info.title,url:info.url}, false, 3); 
            }
        });     
    }
    else {
        var dev_items = [];
        for(var idx in devs) {
            dev_items.push(cm.Item({label:devs[idx].deviceName, data:devs[idx].did}));
        }

        menus_list.menu_selection = cm.Menu({
            label:extLocale["Send to Dolphin"],
            image:data.url("src/images/logo16.png"),
            context: [
                cm.URLContext(["http://*","https://*"]),
                cm.SelectionContext()
            ],    
            contentScript: 'self.on("click", function(node,data) { var text = ""; if(node.tagName.toLowerCase()=="input") { text = node.value.substring(node.selectionStart, node.selectionEnd);}else{ text=window.getSelection().toString(); } if(text.length > 1000) { alert(decodeURIComponent("%E4%B8%8D%E8%83%BD%E8%B6%85%E8%BF%871000%E5%AD%97%E7%AC%A6%E3%80%82")); return;} self.postMessage({dev_id:data,url:"",title:text});})',
            onMessage:function(info) {
                push(info.dev_id,{title:info.title,url:info.url}, false, 3); 
            },
            items: dev_items          
        }); 
    } 
}
