exports.Pusher = Pusher;
exports.SonarPusher = SonarPusher;

var util = require("../util");
var jQuery = util.jQuery();
var urlencode = util.urlencode;
var data = require("sdk/self").data;
var pageWorkers = require("sdk/page-worker");
var timers  = require("sdk/timers");
var setTimeout = timers.setTimeout;
var clearTimeout = timers.clearTimeout;
var sync_control = require("../tab/tabsync").sync_control;
var CommInfo = require("../common").CommInfo();
var print_msg = require("../util").print_msg;
var localStorage = require("sdk/simple-storage").storage;
var reflashPage = require("../main").reflashPage;
var GetPushDevicesSorted = require("../util").GetPushDevicesSorted;
var PushRespInjectBox = require("./push").PushRespInjectBox;
var save_devicename = util.save_devicename;
var debug = require("../util").debug();

/*
	Push API Address.
*/
var PUSH_DOMAIN = "https://pushservice.dolphin-browser.com";
var get_pushURL =  function(){
	if(CommInfo.push_domain) {
		return CommInfo.push_domain + "/data/1/sonar_conf";	
	}
	else {
		return PUSH_DOMAIN + "/data/1/sonar_conf";
	}
}
var getURL = function() {
	if(CommInfo.push_domain) {
		return CommInfo.push_domain + "/data/1/get_msg";
	}
	else {
		return PUSH_DOMAIN + "/data/1/get_msg";
	}
};
var listURL = function() {
	if(CommInfo.push_domain) {
		return CommInfo.push_domain + "/data/1/device/auth";
	}
	else {
		return PUSH_DOMAIN + "/data/1/device/auth";
	}
};
var affirmURL = function() {
	if(CommInfo.push_domain) {
		return CommInfo.push_domain + "/data/1/device/affirm";
	}
	else {
		return PUSH_DOMAIN + "/data/1/device/affirm";
	}
};	

var SONAR_PUSH_URL = "http://sonarpush.dolphin-browser.com/cometd";
/*
	base64 encode/decode key
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
*/

/*
	Received message types.
*/
var RECV_MSG_TYPE={
	'join':0,
	'new':1,
	'del':2,
	'pub':3,
	'mod_dvc_nm':4,
	'remove':5,
	'received':6,
	'offline':7,
	'states':8
};

function Pusher(hickey)
{
	var _dev_id = hickey.userinfo.dev_id;
	var _dev_token = hickey.userinfo.token;
    var page_worker = null;	
    var wsWorker = null;
    var _connected = false;
	var _devicelist;
	var _connect_num = 1;
	var _affirm_retry_num = 1;
	var pub_channel = '';
	var self = this;
    var _connectStatus = false;
	var auth_timeout_id = null;
	
	var connecting = false;
	var pushClientId = 0;
	var ext = "";
	
	var _listener_cometd = {
		publish: function(channel, msg, extInfo) {
			wsWorker.port.emit("message", {category:'push', type:channel, data:msg, extInfo:extInfo});         
		}
    };

	this.isConnect = function()
	{
		return _connected;
	};
	
	this.isConnectingStatus = function()
	{
		return {connecting:connecting, retry_num:_connect_num};
	};
	
	this.ConnectStatus = function() {
		return _connectStatus;
	};
	
	/*
		When message pushed from push-server arrive, this function will be called.
		right now, there are 4 message type,[join, new ,del, pub]
	*/
	function receive(message)
	{
		var msg_type = RECV_MSG_TYPE[message.data.type];
		switch(msg_type)
		{
			case RECV_MSG_TYPE['join']:
				print_msg("[11][Push] push join has a response...");	
				recv_join(message.data);					
				break;
			case RECV_MSG_TYPE['new']:
				recv_new(message);
				break;
			case RECV_MSG_TYPE['del']:
				recv_del(message);
				break;
			case RECV_MSG_TYPE['remove']:
				recv_remove(message);
				break;
			case RECV_MSG_TYPE['pub']:
				recv_pub(message);
				break;
			case RECV_MSG_TYPE['mod_dvc_nm']:
				recv_dev_rename(message);
				break;
			case RECV_MSG_TYPE['received']:
				//recv_pushAffirm(message);
				break;
			case RECV_MSG_TYPE['offline']:
				recv_offline(message);
				break;	
			case RECV_MSG_TYPE['states']:
				recv_join(message.data);
				break;						
			default:
				print_msg('unrecognize msg type');
		}
	}

	function recv_offline(message)
	{
		deviceId=message.data.device_id;
		//update device online/offline status
		//TODO:
		for(var idx in CommInfo.device_list)
		{
			dev = CommInfo.device_list[idx];
			if(dev.did == deviceId)
			{
				dev.state=0;
				//Refresh
				if(CommInfo.popup_id != null)
				{
					//UI display		
					reflashPage(GetPushDevicesSorted(), CommInfo.get_device_shortcut(), 'push');
					//CommInfo.popup_id.reflashPage(GetPushDevicesSorted(),CommInfo.get_device_shortcut(),'push');
				}
				break;
			}
		}
	}
	
	function tab_receive(message) {
		print_msg("receive tab sync push message");
		var data = JSON.parse(message.data.data);
		hickey.handler.tab_sync(data[0].sid);
	}
	
	function bookmark_receive(message) {
		var data = JSON.parse(message.data.data);
		if(data[0].type == 128) {
			print_msg("receive bookmark sync push message");
			hickey.handler.bookmark_sync(data[0].sid);			
		}
	}
	
	/*
		After device join success, server will push a join message back.
		This message may include history data which is pushed to this device 
		when the device was offline.
	*/
	function recv_join(message)
	{
		print_msg("[12][Push] add listener...");	
		subscribe(['push','publish']); 
		subscribe(hickey.subscribe); 
		//self.syncChange(hickey.subscribe);
		
		//right now, push connect is successful, then display devices on main panel.
		_connected = true;
		connecting = false;
		devices= {};
		try
		{
			devices = JSON.parse(message.device_list);							
		}
		catch(e)
		{
			//no code here.
		}		
		/*
		if(devices)
		{
			for(var did in devices)
			{
				if(_deviceList[did])
				{
					_deviceList[did].state = devices[did].state;
				}
			}
		}
		*/
		_deviceList = devices;
		//recheck device connect status.
		if(hickey.handler.device_list != null)
		{
			hickey.handler.device_list(_deviceList);
		}
		
		print_msg("[13][Push] now main panel can show device list...");	
		
		//inform client that there are some old data on server, get it by data
		oldMessage = [];
		try
		{
			if(message.data && message.data != "")
			{
				oldMessage = JSON.parse(message.data);			
			}
		}
		catch(e)
		{
			//no code here.
		}
		
		if(oldMessage.length > 0){
			getHistoryData(oldMessage);			
		}
		//if join success
		sync_control();
	}
	
	/*
		When a new device join.
	*/
	function recv_new(message)
	{
	
		//there is a new client coming, update the device list
		if(_deviceList[message.data.device_id])
		{
			delete 	_deviceList[message.data.device_id];
		}
		deviceInfoJson = JSON.parse(message.data.device_info);
		deviceInfoJson.state=1;		
		_deviceList[message.data.device_id] = message.data.device_info;
		
		var new_device={};

		new_device[message.data.device_id]=deviceInfoJson;
		
		if(hickey.handler.device_add != null)
		{
			hickey.handler.device_add(new_device);
		}		
	}
	
	function del_dev(dev_id)
	{
		if(_deviceList != null) {
			if(_deviceList[dev_id] != null) {
				delete _deviceList[dev_id];					
			}
		}
	}
	
	/*
		When a device is delete by this user.
	*/
	function recv_del(message)
	{
		if(message.data.del_id == _dev_id) {
			hickey.handler.logout();		
			affirm_logout(1);
		}
		else {
			del_dev(message.data.del_id);
			if(hickey.handler.device_remove != null)
			{
				hickey.handler.device_remove(message.data.del_id);
			}		
		}
	}
	
	function recv_remove(message) {
		recv_del(message);
	}
	
	/*
		Normal push data receiver.
	*/
	function recv_pub(message)
	{
		//some client push data to me, show it! remeber decode it 
		//hickey.message_new_handler(jQuery.base64.decode(message.data.data));
		if(hickey.handler.receive_new != null)
		{
			var data = JSON.parse(message.data.data);
			if(data.push_type == 1 || data.push_type == 2) {
				print_msg('');
				hickey.handler.receive_new(data.push_data);				
			}
			else {
				print_msg("receive text:[%d]"+ data.push_data.title,data.push_type);
			}
		}
		_listener_cometd.publish('push', {
			type : 'affirm',
			device_id : _dev_id,
			msgid : message.data.msgid,
			data : message.data.data
		});		
	}
	
	function recv_dev_rename(message)
	{
		if(hickey.handler.device_rename != null)
		{
			//if(message.data.device_info) {}
			hickey.handler.device_rename(message.data.target_id, JSON.parse(message.data.device_info));
		}
	}
	
	this.push = function(destination, pushData, extInfo)
	{
		_listener_cometd.publish('push', {
			type : 'pub', 
			device_id : _dev_id,
			dst_id : destination,
			//data : jQuery.base64.encode(pushData),
			data : pushData
		},
		extInfo
		);
	};

	this.remove = function(device_id)
	{
		del_dev(device_id);
		_listener_cometd.publish('device', {
			type: 'del', 
			device_id: _dev_id,
			del_id: device_id
		});		
	};
	
	this.rename = function(device_id,name,type)
	{
		hickey.userinfo.device_info.name = name;
		_listener_cometd.publish('device', {
			type: 'mod_dvc_nm', 
			device_id: _dev_id,
			target_id: device_id,
			device_info:JSON.stringify({type:type, name:name})
		});		
	};

	function getHistoryData(informDataData)
	{
		informData = {
			"token" : _dev_token,
			"device_id" : _dev_id,
			"msg_id_list" : informDataData
		};
		
		jQuery.ajax({
			url : getURL(),
			type : 'post',
			data :urlencode(informData),

			//call back function, get the old data
			success : function(message){
				if(message.status == 0)
				{				
					//do logic for message
					var pushData = [];
					for(key in message.data)
					{
						//pushData.push(jQuery.base64.decode(message.data[key]));
						//if data is null, error may occur, so, we need filter null object out.
						if(message.data[key] != null)
						{
							pushData.push(message.data[key]);
						}	
					}
					
					if(hickey.handler.receive_old != null)
					{
						hickey.handler.receive_old(pushData);
					}	
				}
			},
            error:function(){ print_msg("get history data error.");}
		});
	}

	function getDeviceList()
	{
		print_msg("[2][Push] getDeviceList...");
		//_connect_num += 1; 
		//used for log statistic
		hickey.userinfo.device_info.browser = "firefox";
		var _auth_token = {
			"token" : _dev_token,
			"device_id" : _dev_id,
			"device_info" : hickey.userinfo.device_info
		};
		
		print_msg(JSON.stringify(_auth_token));
		
		var msg_encode = urlencode(_auth_token);
		print_msg(listURL());
		jQuery.ajax({
			url : listURL(),
			type : 'post',
			data : msg_encode,

			//call back function, get the old data
			success : function(message){
				print_msg("[3][Push] getDeviceList has response..."+JSON.stringify(message));	
				if (message.status == 0){
					print_msg("[4][Push]getDeviceList success ...");	
					_connect_num = 1;
					var device_state = message.data.device_state;
					if(device_state == 0) {
						hickey.handler.sync_setting(message.data.tab_sync_commit_delay*1000);
						_deviceList = message.data.device_list;
						if(message.data.ext) {
							ext = message.data.ext;
						}
						push_url = message.data.url;
						
						//if no other device, do not connect to push server.
						var dev_num = 0;
						for(var id in _deviceList)
						{
							dev_num++;
							break;
						}
						
						if(dev_num > 0)
						{
							push_connect(push_url);						
						}
						else
						{
							_connected = true;
							//UI display
							reflashPage([], {}, 'push');
							
							save_devicename(hickey.userinfo.device_info.name);	
							//start sync.
							sync_control();	
						}					
					}
					else{
						print_msg("[5][Push] device is logout in webmanagement ...");	
						hickey.handler.logout();
						_affirm_retry_num = 1;
						affirm_logout(device_state);
					}
				}
				else
				{
					//logout
					if(message.status == 11)
					{
						hickey.handler.logout();	
					}
					//if failed, will try 10 times.
					if(_connect_num < 1024){
						_connect_num = _connect_num * 2;
						print_msg("retry get device list [%d]",_connect_num);
						auth_timeout_id = setTimeout(getDeviceList, 1000*_connect_num);
					}
					else{
						_connect_num = 1;
						_connected = false;
						connecting = false;						
						print_msg("get device list failed.");
					}
				}
			},
			error: function(){
				//if failed, will try 10 times.
				if(_connect_num < 1024){
					_connect_num = _connect_num * 2;
					print_msg("retry get device list [%d]",_connect_num);
					auth_timeout_id = setTimeout(getDeviceList, 1000*_connect_num);
				}
				else{
					_connect_num = 1;
					_connected = false;
					connecting = false;
					print_msg("get device list failed.");
					
				}
			}
		});				
	}
	      
	function affirm_logout(state) {
		var _auth_token = {
				"token" : _dev_token,
				"device_id" : _dev_id,
				"device_state" : (state == null) ? 1 : state
		};
		
		var msg_encode = urlencode(_auth_token);	
		
		jQuery.ajax({
			url : affirmURL(),
			type : 'post',
			data : msg_encode,
			//call back function, get the old data
			success : function(message) {
				if(message.status == 0) {
					//hickey.handler.logout();					
				}
				else
				{
					if(_affirm_retry_num < 1024) {
						_affirm_retry_num = _affirm_retry_num * 2;
						setTimeout(affirm_logout, _affirm_retry_num*1000);				
					}
					else {
						_affirm_retry_num = 1;
						print_msg("affirm logout failed");
					}
				}
			},
			error: function() {
				if(_affirm_retry_num < 1024) {
					_affirm_retry_num = _affirm_retry_num * 2;
					setTimeout(affirm_logout, _affirm_retry_num*1000);				
				}
				else {
					_affirm_retry_num = 1;
					print_msg("affirm logout failed");
				}
			}
		});		
	}	      
	      
	      
	this.login = function()
	{ 		
		print_msg("[1][Push] login Start...");
		
        _connected = false;
		if(wsWorker == null) {
			create_pageWorker();		
		}
		connecting = true;		
        _connect_num = 1;
        getDeviceList();
	};

    function create_pageWorker() {
    	var data = require("sdk/self").data;
    	print_msg("create_pageWorker");
    	
        wsWorker  = pageWorkers.Page({
            contentUrl: data.url("src/worker.html"),
            contentScriptFile: [
				data.url("src/thirdPartylibs/jquery/jquery-1.7.2.js"),
				data.url("src/thirdPartylibs/org/cometd/cometd-namespace.js"),
				data.url("src/thirdPartylibs/org/cometd/cometd-json.js"),
				data.url("src/thirdPartylibs/org/cometd/Utils.js"),
				data.url("src/thirdPartylibs/org/cometd/Transport.js"),
				data.url("src/thirdPartylibs/org/cometd/RequestTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/TransportRegistry.js"),
				data.url("src/thirdPartylibs/org/cometd/WebSocketTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/LongPollingTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/CallbackPollingTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/Cometd.js"),
				data.url("src/thirdPartylibs/jquery/jquery.cometd.js"),
				data.url("src/ext_libAdaptor.js")
            ]    
        });    
        
        wsWorker.on("message", function(msg){
            var category = msg.category;
            var type = msg.type;
            var data = msg.data;
            if(type == 'recv') {
                receive(data);
            }
            else if (type == 'tab') {
            	tab_receive(data);
            }
            else if(type == 'bookmark') {
            	bookmark_receive(data);
            }
            else if(type == 're_auth') {
            	try{
            		self.disconnect();
            	}
            	catch(e) {
            		print_msg("re_auth disconnect failed.");
            	}
            	self.login();
            }
            else if(type == 'connect') {
            	print_msg("receive connect message.");
            	_connectStatus = data.status;
				
				//save current connect clientId.
				if(data.clientId) {
					localStorage['DolphinBrowserPushClientId'] = data.clientId;
				}
            }
			else if(type == 'multiple-clients') {
				print_msg("multiple-clients detected");
				try{
					clientId = localStorage['DolphinBrowserPushClientId'];
            		self.disconnect({clientId:clientId});
            	}
            	catch(e) {
            		print_msg("re_auth disconnect failed.");
            	}
				self.login();
			}
			else if(type =='inject')
			{
				PushRespInjectBox(data.type, data.subtype, data.status);
			}
			else if(type == 'rejoin')
			{
				//PushRespInjectBox(data.type, data.subtype, data.status);
			}
		});
    }
       
    //send message to contentscript to connect with push server. 
	function push_connect(push_url)
    {
		var sync_types = hickey.subscribe;
		var channel_types = channelName2TypeMapping(sync_types);
		if(wsWorker != null) {
			wsWorker.port.emit("message", {category:'push', type:'connect', data:{url:push_url, dev_id:_dev_id, sync_channels:channel_types, token: _dev_token, ext: ext, debug:debug, version:CommInfo.client_version}});  		
		} 
    }

	function channelName2TypeMapping(sync_types) {
		var channel_types = 0;
		for(var id in sync_types) {
			if(sync_types[id] == 'tab') {
				channel_types += 2;
			}
			else if(sync_types[id] == 'bookmark') {
				channel_types += 8;
			}
		}
		
		return channel_types;
	}
	
	this.syncChange = function(current) {
		var channel_types = 0;
		if(current != null) {
			channel_types = channelName2TypeMapping(current);		
		}
		
		try{
			_listener_cometd.publish('device', {
				type: "mod_sync_channel",
				device_id: _dev_id,
				sync_channels: channel_types
			});		
		}
		catch(e) {
			print_msg("syncChange Push exception:" + e);
		}
	}
	
	function subscribe (config) {
		print_msg("subscribe channel.");
		if(wsWorker != null) {
			wsWorker.port.emit("message", {category:'push', type:'subscribe', data:config});  
		}	
	}
	
	function unSubscribe (config) {
		if(wsWorker != null) {
			wsWorker.port.emit("message", {category:'push', type:'unSubscribe', data:config});  		
		}
	}
	
	this.disconnect = function(param)
	{
		if(auth_timeout_id != null) {
			try{
				clearTimeout(auth_timeout_id);
			}
			catch(e) {
				print_msg(e);
			}
			auth_timeout_id = null;
		}
	    _connected = false;
   	 	_connectStatus = false;	    
		
		if(wsWorker != null) {
			self.syncChange();
			wsWorker.port.emit("message",{category:'push', type:'disconnect', data:param});
			wsWorker.destroy(); 
			wsWorker=null;			
			
			//later we need free the pageworker resources.
			/*
			setTimeout(function(){ 
				try {
					wsWorker.destroy(); 
					wsWorker=null;
				}
				catch(e){
					print_msg(e);
				}	
			}
			, 1000);
			*/
		}
	};
}

/*
	Push module for sonar login
*/
function SonarPusher(hickey)
{
    var wsWorker = null;
	
	this.login = function()
	{
     	create_pageWorker();   	
		//connect to push server.
		push_connect(SONAR_PUSH_URL);						
	}
	
	function push_connect(push_url)
    {
        wsWorker.port.emit("message", {category:'sonar', type:'connect', data:{url:push_url, dev_id:hickey.dev_id }}); 
    }

    function create_pageWorker(){
        wsWorker  = pageWorkers.Page({
            contentUrl: data.url("src/worker.html"),
            contentScriptFile: [
				data.url("src/thirdPartylibs/jquery/jquery-1.7.2.js"),
				data.url("src/thirdPartylibs/org/cometd/cometd-namespace.js"),
				data.url("src/thirdPartylibs/org/cometd/cometd-json.js"),
				data.url("src/thirdPartylibs/org/cometd/Utils.js"),
				data.url("src/thirdPartylibs/org/cometd/Transport.js"),
				data.url("src/thirdPartylibs/org/cometd/RequestTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/TransportRegistry.js"),
				data.url("src/thirdPartylibs/org/cometd/WebSocketTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/LongPollingTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/CallbackPollingTransport.js"),
				data.url("src/thirdPartylibs/org/cometd/Cometd.js"),
				data.url("src/thirdPartylibs/jquery/jquery.cometd.js"),
				data.url("src/ext_libAdaptor.js")
            ]    
        });
        wsWorker.on("message", function(msg){
            var category = msg.category;
            var type = msg.type;
            var data = msg.data;
            if(category == 'sonar' && type == 'recv') {
                recv(data);
            }            
        });
    }

	function recv(msg)
	{
		if(hickey.handler)
		{
			var user_info = msg.data.data;
			hickey.handler(JSON.parse(user_info));
		}		
	}    	
	
	this.disconnect = function()
	{	
		print_msg("push disconnect.");
	    _connected = false;
        wsWorker.port.emit("message",{category:'sonar', type:'disconnect'});
        wsWorker.destroy();
        wsWorker=null;
        
        //later we need free the pageworker resources.
        //timers.setTimeout(function(){wsWorker.destory(); wsWorker=null;}, 1000);
	};		
}
