exports.API_sync_state = API_sync_state;
exports.API_tab_sync_create = API_tab_sync_create;
exports.API_tab_sync_update = API_tab_sync_update;
exports.API_validate_token = API_validate_token;

var  jQuery = require("../util").jQuery();
var API = require("../const").API();
var CommInfo = require("../common").CommInfo();
var localStorage = require("sdk/simple-storage").storage;
var util = require("../util");
var get_clientid = util.get_clientid;
var get_devicename = util.get_devicename;
var urlencode = util.urlencode;
var showNotification = util.showNotification;
var get_open_tabs = require("./tabsync").get_open_tabs;
var reflashPage = require("../main").reflashPage;
var track_event = require("../track").track_event;
var save_user_login_info = require("../login/account").save_user_login_info;
var setBelugaCode = require("../track").setBelugaCode;
var SID = {
	current : 0,
	ongoing : 0
};

var print_msg = require("../util").print_msg;
/*
	Get user id
*/
function API_validate_token()
{
	var data={'token':CommInfo.token};
	var data_encode = urlencode(data);
	print_msg(JSON.stringify(data_encode));
	print_msg(API.validate_token);
	
	jQuery.ajax({
		url:API.validate_token,
		contentType:"application/json",
		type:"post",
		data:data_encode,
		headers:{"ClientVersion":"chromeExtv1.0"},
		success:CALLBACK_validate_token,
		error:function(){
			print_msg("validate token error.");
		}
	});	
}

/*
	Callback of API_validate_token.
	Params:
		xhr:XMLHttpRequest.
*/
function CALLBACK_validate_token(resp)
{
	if(resp.status == 0) 	//connect success
	{
		CommInfo.user_id  = resp.data.user_id;
		save_user_login_info();
		setBelugaCode(CommInfo.user_id);
	}
}


/*
	Get sync state
*/
function API_sync_state()
{
	track_event({
		category:'general',
		action:'TabSync',
		label:'trigger',
		value:1
	});		

    print_msg("get state");
	var data={'token':CommInfo.token};
	var data_encode = urlencode(data);

	jQuery.ajax({
		url:API.sync_state(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		headers:{"ClientVersion":"chromeExtv1.0"},
		success:CALLBACK_save_latest_sync_id,
		error:function(){
			print_msg("get state error.");
			track_event({
				category:'general',
				action:'TabSync',
				label:'fail',
				value:1
			});	
		}
	});
}

/*
	Callback of API_sync_state.
	Params:
		xhr: XMLHttpRequest
*/
function CALLBACK_save_latest_sync_id(resp)
{
    if(resp.status == 0) 	//connect success
    {	
        //tab
        var latest_sid = resp.data.latest_tab_sid;
        var	after_sid = CommInfo.after_sid();		
        print_msg('server_last_tab_id:'+latest_sid);
        print_msg('local_last_tab_id:'+after_sid['tab']);
        if(after_sid['tab'] < latest_sid)
        {
			SID.current = after_sid['tab'];
			SID.ongoing = after_sid['tab'];
            API_sync_getchunk({type:'tab',latest_sid:latest_sid});
            CommInfo.delete_tabs.splice(0,CommInfo.delete_tabs.length);
        }
        else
        {
            get_open_tabs();
        }
    }
    else
    {
        print_msg("Sync tab failed.");
        //showNotification('Sync Failed','Sync failed!');
		track_event({
			category:'general',
			action:'TabSync',
			label:'fail',
			value:1
		});	        
    }
}

/*
	Send get chunk request.
	Params:
		context:sync item information, json dict
*/
function API_sync_getchunk(context)
{		
	try{
		var data = {};
		var token = CommInfo.token;
		var after_sid = SID.ongoing;//CommInfo.after_sid()[context.type];
		var latest_sid = context.latest_sid;
		var limit = latest_sid - after_sid;
		var type = get_type(context);
		
		data.token=token;
		data.after_sid = after_sid;
		data.limit = limit>20?20:limit;
		data.type = type;
		data.cid = get_clientid(false);
		
		if(SID.current == 0) {
			data.no_deleted = 1;
		}
		
		var data_encode=urlencode(data);
		
		
		jQuery.ajax({
			url:API.sync_getchunk(),
			contentType:"application/json",
			type:"post",
			data:data_encode,
			headers:{"ClientVersion":"firefoxExtv1.0"},
			success:CALLBACK_getchunk,
			error:function(){
				print_msg("get chunk error.");
				track_event({
					category:'general',
					action:'TabSync',
					label:'fail',
					value:1
				});			
			}
		});
	}catch(err){
		print_msg(err);
	}
}

/*
	Callback of getchunk.
	This function save the obj get from server to local storage,
	if get chunk is not finish, getchunk will be called again.
	
	Params:
		xhr: XMLHttpRequest.
*/
function CALLBACK_getchunk(resp)
{
		if(resp.status == 0) 	//connect success
		{	
			//tab
			var latest_sid = resp.data.latest_sid;
			var chunk_latest_sid = resp.data.chunk_latest_sid;
			var updated_objs = resp.data.updated_objs;
			
			CommInfo.delete_tabs = CommInfo.delete_tabs.concat(resp.data.deleted_ids);
			
			var tabs = CommInfo.tab_sync_list();
			for(var idx in updated_objs)
			{
				var updated_obj = updated_objs[idx];
				//Device having no tab will be filtered out.  
				if(updated_obj.payload && updated_obj.payload.tabs)
				{
					if(updated_obj.payload.tabs.length > 0) {
						tabs[updated_obj._id] = {name:updated_obj.payload.name, data:updated_obj.payload.tabs, sid:updated_obj.sid, dev_id:updated_obj.cid};
					
						if(updated_obj.payload.hasOwnProperty('devicetype'))
						{
							tabs[updated_obj._id].dev_type = updated_obj.payload.devicetype;
						}
					}
					else {
						//if update device has no tabs, delete it from localStorage.
						if(tabs.hasOwnProperty(updated_obj._id)) {
							delete tabs[updated_obj._id];
						}
					}
				}
			}
			
			//save other device sync result to localStorage, which will be displayed in index.html later.
			localStorage['DolphinBrowserSyncTabs'] = JSON.stringify(tabs);
	
			
			if(chunk_latest_sid < latest_sid)
			{
				SID.ongoing = chunk_latest_sid;
				API_sync_getchunk({type:'tab',latest_sid:latest_sid});
			}
			else
			{
				SID.current = chunk_latest_sid;
				SID.ongoing = chunk_latest_sid;
				
				var after_sid = CommInfo.after_sid();
				
				//get tab after_sid
				after_sid['tab'] = chunk_latest_sid;
				
				//save after_sid
				var after_sid_str = localStorage['DolphinBrowserAfterSid'];
				var after_sid_json;
				if(after_sid_str != null && after_sid_str != "")
				{
					after_sid_json = JSON.parse(after_sid_str);
				}
				else
				{
					after_sid_json={};
				}
				
				after_sid_json[CommInfo.user_name] = after_sid;
				localStorage['DolphinBrowserAfterSid'] = JSON.stringify(after_sid_json);
				
				for(var i in CommInfo.delete_tabs)
				{
					CommInfo.tab_sync_list_remove(CommInfo.delete_tabs[i]);
				}
				
				//Refresh popup window.
				var tab_sync_list = CommInfo.tab_sync_list_sorted();
				reflashPage(tab_sync_list, null, 'sync');
				
				get_open_tabs();
			}
		}
		else
		{
            print_msg("sync failed.");
 			track_event({
				category:'general',
				action:'TabSync',
				label:'fail',
				value:1
			});	
		}
}

/*
	Update the device current open tabs 
	Params:
		sync_id:user sync_id
*/
function API_tab_sync_update(sync_id)
{
	var data={"type":2,"folder":0,"order":1, "pid":"", "cid":get_clientid(false),"_id":sync_id,"deleted":0};
	var payload={"name":get_devicename(),"tabs":CommInfo.cur_tab_list,'devicetype':2};
	data.payload=payload;
	var body={'token':CommInfo.token,'data':[data],"device_id":get_clientid(true)};
	var data_encode=urlencode(body);

	jQuery.ajax({
		url:API.sync_create(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		headers:{"ClientVersion":"chromeExtv1.0"},
		success:CALLBACK_save_tab_id,
		error:function(){
			print_msg("tab sync update error.");
			track_event({
				category:'general',
				action:'TabSync',
				label:'fail',
				value:1
			});	
		}
	});
}

/*
	Display sync notification.
	Params:
		xhr: XMLHttpRequest.
*/
function sync_finish_notify(resp)
{
    if(resp.status == 0)
    {
            print_msg("tab update success.");
    }
    else
    {
            print_msg("tab update failed.");
    }
}	

/*
	Create a new tab id, this means first sync for this device.
*/
function API_tab_sync_create()
{
	var data={"type":2,"folder":0,"order":1, "pid":"", "cid":get_clientid(false),"deleted":0};
	var payload={"name":get_devicename(),"tabs":CommInfo.cur_tab_list,'devicetype':2};
	data.payload=payload;
	var body={'token':CommInfo.token,'data':[data],"device_id":get_clientid(true)};
	var data_encode=urlencode(body);
        
	jQuery.ajax({
		url:API.sync_create(),
		contentType:"application/json",
		type:"post",
		data:data_encode,
		headers:{"ClientVersion":"chromeExtv1.0"},
		success:CALLBACK_save_tab_id,
		error:function(){
			print_msg("tab sync create error.");
			track_event({
				category:'general',
				action:'TabSync',
				label:'fail',
				value:1
			});	
		}
	});
}

/*
	Save sync id to local storage
	Params:
		xhr: XMLHttpRequest.
*/
function CALLBACK_save_tab_id(resp)
{	
    if(resp.status == 0) 	//connect success
    {	
        var tab_sync_id = resp.data[0]._id;
        var sync_id_s = {};
        sync_id_s[CommInfo.user_name] = tab_sync_id;
        localStorage["DolphinBrowserTabSyncId"]=JSON.stringify(sync_id_s);	
        print_msg("tab sync create success.");
		track_event({
			category:'general',
			action:'TabSync',
			label:'success',
			value:1
		});	        
    }
    else
    {
        print_msg("tab sync createb failed.");
		track_event({
			category:'general',
			action:'TabSync',
			label:'fail',
			value:1
		});	        
    }
}

/*
	Get sync item type number
	Params:
		context: sync item information
*/
function get_type(context)
{
	if(context.type == 'bookmark')
		return 128;
	else if(context.type == 'tab')
		return 2;
	else if(context.type == 'history')
		return 4;
}
